function X = invtransparas(theta)
%Transformation of parameters to the unconstrained specification for optimization- translated from
%Lubik-Schorfheide (2004) Gauss code
%The transformation makes new values to lie in [-inf,inf].

%parameter order
%[tau rstar kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, m_eta_r, m_eta_g, m_eta_z, m_eta_e]';

partype=[2;2;2;2;2;1;1;1;2;2;2;0;0;0;2;2]; %parameter type: 1 - constrained as [a,b], e.g. for rho_gz
% a=-1, b=1; 2 - constrained as [o, inf]; 0 - unconstrained

%column 1 - a, column 2 - b, column 3 - c
abc=[1e-05,0,1;1e-05,0,1;1e-05,0,1;1e-05,0,1;1e-05,0,1;1e-05,0.99,1;1e-05,0.99,1;1e-05,0.99,1;1e-05,0,1;1e-05,0,1;1e-05,0,1;0,0,0;0,0,0;0,0,0;1e-05,0,1;1e-05,0,1];

ntheta=length(theta);

for i=1:ntheta
    a=abc(i,1);
    b=abc(i,2);
    c=abc(i,3);
    
    if partype(i,1)==1
        cx=2*(theta(i)-0.5*(a+b))/(b-a);
        theta(i)=(1/c)*cx/sqrt(1-cx^2);
    elseif partype(i,1)==2
        theta(i)= b + (1/c)*log(theta(i)-a);
    end
end
X=theta;
end