%%% This file reproduces the results in the fifth column of Tables S14 and S15:
%%% i) the parameter vector minimizing the KL criterion between the
%%% default specification of the Smets and Wouters (2007) model under
%%% determinacy and the parameters in the space outside the excluded 
%%% neighborhood with c=0.5, using the L1 norm. The adjustment cost parameter is held fixed at
%%% the default value of 5.74. 
%%% ii) the value of the minimized KL criterion and the empirical distances 
%%% for samples of size 80, 150, 200 and 1000.

% The required data files and subroutines are located in: 
%\SW_Identification\
%(swsolv.m, setsimsmat.m, eqvarshock.m, ssdpar.m klsw.m,pfhsw.m,qsw.m,vfhsw.m, 
% theta0swd.mat,true_spectrum0swd.mat )
%Additional files for business cycle frequencies only:
%(klswbc.m,pfhswbc.m,qswbc.m,vfhswbc.m,true_spectrum0swdbc.mat)
%\SW_Identification\Constraints\
%(constraintswadj.m)
%\SW_Identification\Objectives\
%(kloptsw2.m)
%\General\gensys_mod
%(gensys_mod.m, svdrr.m)
%\General\gensys should also be on the Matlab path
%\General\GA_optim.m, PSO_optim.m, psout.m

%The total runtime for GA+Multistart algorithm is about 4.5 hours, and about 7.8 hours for PSO+Multistart  
%algorithm on a Xeon E5-2665 8-core 2.4Ghz processor, using Matlab R2015a.
%%
clear
%% Algorithm selection and Matlab versions
%Select one procedure:
runga=0; %1 if using Genetic algorithm + Multistart combination, 0 otherwise
runpso=1; %1 if using Particle Swarm + Multistart combination, 0 otherwise
          %-----Note PSO is available only in R2014b and later----
 
%Specify the constraint handling for the Genetic algorithm.
NonlinCon='penalty'; %this is the preferred option, available in the matlab version 2014b and later; for earlier versions, set to 'auglag'.
%% Set up parallel computation

 numcore=8; %specify the number of cores
% 
% %for versions before R2014a
% matlabpool open local numcore
% 
% %for versions R2014a and above
ppool=parpool('local',numcore); %create parallel pool object
%% Load parameter values and bounds
 
load theta0swd %load default parameter vector (theta0)

% Set lower and upper bounds
%Parameter order: 
% ['cgy           ';'cmaw          ';'cmap          ';'calfa         ';'czcap         ';
%     'csadjcost     ';'csigma        ';'chabb         ';'cfc           ';'cindw         ';
%     'cprobw        ';'cindp         ';'cprobp        ';'csig          ';'crpi          ';
%     'crdy          ';'cry           ';'crr           ';'crhoa         ';'crhob         ';
%     'crhog         ';'crhoqs        ';'crhoms        ';'crhopinf      ';'crhow         ';
%     'sigma_a       ';'sigma_b       ';'sigma_g       ';'sigma_qs      ';'sigma_m       ';
%     'sigma_pinf    ';'sigma_w       ';'gambar        ';'100(1/cbeta-1)'];

%Same as SW, except: LB of Taylor rule inflation weight up to 1.1, and UB of
%the MA parameters cmaw and cmap reduced to 0.9, UB of AR parameters crhoa,
%crhob, crhog, crhoqs, srhoms, crhopinf and crhow reduced to 0.99.
lb=[0.010;0.010;  0.010;  0.010;0.010;2;0.250;0.0010;1;0.010;0.30;0.010;0.50;0.250;1.1;0.010;0.010;0.50;0.010;0.010;0.010;0.010;0.010;0.010;0.0010;0.010;0.0250;0.010;0.010;0.010;0.010;0.010;0.10;0.010]';
ub=[2;    0.90;0.90;1;1;15;3;0.99000;3;0.99000;0.95000;0.99000;0.95000;10;3;0.50000;0.50000;0.97500;0.990;0.990;0.990;0.990;0.990;0.990;0.990;3;5;3;3;3;3;3;0.80000;2]';

%exclude the adjustment cost parameter (csadjcost)
lb(6)=[];
ub(6)=[];
%% Select frequencies 
bc=0; %0 for full spectrum; 1 for business cycle frequencies only

% specify constraints 
ns=0.5; %size of excluded neighborhood; only needed for constrained case.

nrm=1; %set norm for the constraint function (1,2, or Inf)
  
numpar=length(lb);%number of parameters in the objective function
indp=[1:numpar]; %vector of parameter indices to be constrained
wgt=ones(1,length(indp)); %vector of weights for the constraint function

%% Detailed specifications; you should not have to modify them

if bc==0
    n=500; %number of points to evaluate the integral
    w=2*pi*(-(n/2-1):1:n/2)'/n; %form vector of Fourier frequencies
    resfilename=['table_S1415_p5'];
elseif bc==1
    n=1900; %number of points to evaluate the integral
    w=2*pi*(-(n/2-1):1:n/2)'/n; %form vector of Fourier frequencies
    resfilename=['table_S1415_p5_bc'];
end



con1=[0,ns]; %constraint handling for GA and Multistart
 
con2=[1,ns]; %set constraint handling for Particle Swarm optimization.

ObjectiveFunction = @(theta0)kloptsw2(theta0,con1,w,wgt,nrm,indp,bc); %set objective function for GA/multistart
ObjectiveFunctionP = @(theta0)kloptsw2(theta0,con2,w,wgt,nrm,indp,bc); %set penalized objective function for PSO
ConstraintFunction=@(xest)constraintswadj(xest,ns,wgt,nrm,indp); %set constraint for the problem
 
dispalg='iter'; %set whether algorithm iterations are displayed.
dispint=20; %interval between displayed iterations (for PSO)
 
%% GA algorithm settings
if runga==1
gen=1000; %max number of generation for GA
stgenlim=50; %max number of stall generations (i.e., no improvement found)

initpop=[]; %set initial population (if smaller than popsize, MATLAB will
%randomly draw the rest. If [], the whole population is randomly drawn.
%Can be a row vector of dimension numpar or a matrix. Each row of the
%matrix is then a candidate initial value.

popsize=300; %population size
elcnt=3; %elite count - number of elite individuals retained in population

tolfunga=1e-10; %tolerance level for improvement in the objective for GA
tolconga=1e-10; %tolerance level for constraint for GA

usepga=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []
%In later versions of Matlab, 1 and 0 can also be used respectively. 
end

%% PSO algorithm settings
if runpso==1
    
swarmsize=1000; %swarm size (similar concept to population size for GA)
maxitpso=1000; %maximum number of iterations (similar concept to generations for GA)
stiterlim=100; %max number of stall PSO iterations (i.e., no improvement found)
initswarm=[]; %set initial population (if smaller than swarmsize, MATLAB will
%randomly draw the rest. If [], the whole population is randomly drawn.
%Can be a row vector of dimension numpar or a matrix. Each row of the
%matrix is then a candidate initial value.

minfn=0.1; %smallest fraction of neighbors for PSO (smallest size of the 
%adaptive neighborhood)

tolfunpso=1e-10; %tolerance level for improvement in the objective for PSO

psoname=['psoswadjl1_c',num2str(ns*100)]; %set name for a temp output file that stores the swarms (problem-based)
OutFun=@(optimValues,state)psout(optimValues,state,psoname); %output 
%function for extracting swarms from PSO runs for further local optimization.

useppso=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []


end
%% Multistart algorithm settings

numrpoints=50; %number of random starting points for Multistart

usepms=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []

% settings for fmincon
maxit=1000; % set max number of iterations
maxfev=20000; % set max number of function evaluations
tolfunfmc=1e-10; %tolerance level for improvement in the objective for fmincon
tolconfmc=1e-10; %tolerance level for constraint for fmincon
tolx=1e-10; %tolerance on solution value

localg='active-set'; %set main local algorithm to be used for multistart

%% Run optimization

if runga==1
    timega=tic;
    GA_optim %run GA+Multistart
    timelga=toc(timega); %time taken by GA/Multistart
    save(resfilename) %save intermediate results
end
save(resfilename)
if runpso==1
    timepso=tic;
    PSO_optim %run PSO+Multistart
    timelpso=toc(timepso); %time taken by PSO/Multistart
    save(resfilename) %save intermediate results
end



%% Arrange results
values=[]; %blank for storing best function values
solvecs=[]; %blank for stroing solution vectors
if runga==1
    values=[values;fvalga;fvalga2];
    solvecs=[solvecs;xestga;xestga2];
end
if runpso==1
    values=[values;fvalpso;fvalpso2];
    solvecs=[solvecs;xestpso;xestpso2];
end

err=find(values<0);
values(err)=1e07; %penalize negative values that may occur due to algorithm error
indm=find(values==min(values)); %minimum value(s)
indm=indm(1); 
temp1=['thetaind=[solvecs(',num2str(indm),',(1:5)),theta(6),solvecs(',num2str(indm),',(6:end))]'';']; %parameter vector
temp2=['kl=values(',num2str(indm),')/10000;']; 
eval(temp1); %save final parameter result
eval(temp2); %save minimized KL distance
%%
%% Empirical distance computation
if bc==0
    ed=[pfhsw(theta,thetaind,1,1,0.05,80);pfhsw(theta,thetaind,1,1,0.05,150);pfhsw(theta,thetaind,1,1,0.05,200);pfhsw(theta,thetaind,1,1,0.05,1000)];
elseif bc==1
    ed=[pfhswbc(theta,thetaind,1,1,0.05,80);pfhswbc(theta,thetaind,1,1,0.05,150);pfhswbc(theta,thetaind,1,1,0.05,200);pfhswbc(theta,thetaind,1,1,0.05,1000)];
end
%% Print and save results
par=['cgy           ';'cmaw          ';'cmap          ';'calfa         ';'czcap         ';
    'csadjcost     ';'csigma        ';'chabb         ';'cfc           ';'cindw         ';
    'cprobw        ';'cindp         ';'cprobp        ';'csig          ';'crpi          ';
    'crdy          ';'cry           ';'crr           ';'crhoa         ';'crhob         ';
    'crhog         ';'crhoqs        ';'crhoms        ';'crhopinf      ';'crhow         ';
    'sigma_a       ';'sigma_b       ';'sigma_g       ';'sigma_qs      ';'sigma_m       ';
    'sigma_pinf    ';'sigma_w       ';'gambar        ';'100(1/cbeta-1)'];
t0=num2str(theta,3);
ti=num2str(thetaind,3);
t3=['KL    ';'T=80  ';'T=150 ';'T=200 ';'T=1000'];

for i=1:34
    sp(i,:)='  ';
end
disp 'Table S14. Parameter values minimizing the KL criterion, SW (2007) model'
disp '        (b) Adj. cost fixed, L1 norm'
disp '           theta0     c=0.5'
disp([par,sp,t0,sp,ti])

disp 'Table S15. KL and empirical distances between theta_c and theta_0, SW (2007) model'
disp '        (b) Adj. cost fixed, L1 norm'
disp '           c=0.5'
disp([t3,sp(1:5,:),num2str([kl;ed],3)])
save(resfilename)
%% Close parallel pool

% %for versions before R2014a
% matlabpool close
% 
% %for versions R2014a and above
delete(ppool) %delete the parallel pool object
