function [c, ceq] = constraintswgbcl(xest,ns,wgt,nrm,indp)
%Constraint function for global identification under determinacy (SW (2007) model)
%Keeps the distance between two parameter vectors >= ns

%Steady state growth rate, discount factor, Taylor rule inflation coefficient 
% and labor supply elasticity fixed in optimization.

%Inputs:
%xest - 1x(p+q-4) candidate vector of parameters
%ns - neighborhood size (denoted "c" in the paper)
%wgt - 1x(p+q-4) user-specified vector of weights
%nrm - specified norm, takes inputs 1 (for L-1 norm), 2(for L-2 norm) and Inf (for infinity norm)
%indp - vector of indices for parameters to be constrained, in increasing
%order. Note here the parameter order has to take into account parameters
%fixed in optimization. E.g., if the first parameter is fixed at
%its default value in optimization, index 1 will correspond to the next
%free parameter.

load theta0swd; %load benchmark model parameters
xcon=xest(indp);
thetain=[theta(1:13);theta(16:32)]'; %parameters fixed in optimization removed
   c = ns-norm((xcon-thetain(indp))./wgt,nrm);
   ceq = [];
end