function X = kloptsw(theta0,con,w,wgt,nrm,indp,bc)

%Function to compute the KL distance between models for distance minimization
%in SW(2007) model.
%All parameters can vary.

%con=[pen,c] 
%pen=1 triggers the penalty in case of constraint violation
%c denotes the size of excluded neighborhood
%w - vector of frequencies between [-pi,pi]
%bc=1 then the function uses only the business cycle frequencies in
%computation; otherwise the full spectrum is used.

%wgt,nrm,indp - these inputs are needed for evaluating the constraint
%function should penalty need to be applied. 
warning('off','all');
nw=length(w);

%evaluate constraint and apply flat penalty if violated
if con(1)==1
    conval=constraintsw(theta0,con(2),wgt,nrm,indp);
if conval>1e-10
    X=1e10;
    return
end
end

  
%% solve the model

ny=7; %number of variables
[TT,TC,TEPS,RC] = swsolv(theta0,1); 
if isempty(TT)
    disp('empty solution: nonexistence/numerical issue') 
    X=1e10;
    return
end
neq=size(TT,2); %number of equations
dck=rcond(eye(neq)-TT);

if dck<1e-10
    X=1e10;
    return
end

if RC==[1;1] %determinacy
     retcode = 0;
   RR = [TEPS, zeros(neq,1)];
 
elseif RC==[1;0] %indeterminacy
    X=1e10;
    return
else%no equilibrium exists/numerical problems. 
    X=1e10;
    return
end

matB=eye(neq);
matA=matB(18:24,:); %observables selection matrix

if bc==0 %full spectrum

    load true_spectrum0swd %benchmark model spectrum (full)

%preparations to compute spectrum
term1=0;  
term2=0;
%tempeye=eye(neq);

% [-49,-1] and [1,49]
for i=1:(nw/2-1); %compute spectrum
exe=exp(-1i*w(i));

mat1=(eye(neq)-TT*exe)\eye(neq);
mat2=mat1';  %note that ' gives the conjugate transpose
spec=matA*mat1*RR*RR'*mat2*matA'/(2*pi); %compute spectrum using representation in the paper

zz=spec\true_spectrum(((i-1)*ny+1):i*ny,:);

term1=term1-2*log(det(zz)); 
term2=term2+2*trace(zz);

end

%the zero frequency
i=nw/2;    
exe=exp(-1i*w(i));

mat1=(eye(neq)-TT*exe)\eye(neq);
mat2=mat1';  %note that ' gives the conjugate transpose
spec=matA*mat1*RR*RR'*mat2*matA'/(2*pi); %compute spectrum using representation in the paper
zz=spec\true_spectrum(((i-1)*ny+1):i*ny,:);

term1=term1-log(det(zz)); 
term2=term2+trace(zz);
%the last frequency    
i=nw;
exe=exp(-1i*w(i)); 

mat1=(eye(neq)-TT*exe)\eye(neq);
mat2=mat1';  %note that ' gives the conjugate transpose
spec=matA*mat1*RR*RR'*mat2*matA'/(2*pi); %compute spectrum using representation in the paper
zz=spec\true_spectrum(((i-1)*ny+1):i*ny,:);

term1=term1-log(det(zz)); 
term2=term2+trace(zz);

WL=real(term1+term2); %small complex residual of order e-15i sometimes remains

X=WL/(2*nw)-3.5; %KL

elseif bc==1 %bc frequencies only
    load true_spectrum0swdbc %benchmark model spectrum (bc only) 
    
    %identify BC frequencies
    ind1=find(w>=-pi/3 & w<=-pi/16);
    ind2=find(w>=pi/16 & w<=pi/3);
    idbc=[ind1;ind2];

%update to BC frequencies for computation
w2=w(idbc);
n2=length(w2);
term1=0;  
term2=0;
for i=1:n2; %compute spectrum
exe=exp(-1i*w2(i));

mat1=(eye(neq)-TT*exe)\eye(neq);
mat2=mat1';  %note that ' gives the conjugate transpose

ftheta=matA*mat1*RR*RR'*mat2*matA'/(2*pi); %compute spectrum using representation in the paper

term1=term1-log(det(ftheta\true_spectrum(((i-1)*ny+1):i*ny,:))); %second term in the KL


term2=term2+trace(ftheta\true_spectrum(((i-1)*ny+1):i*ny,:)); %first term in the KL

end
WL=real(term1+term2); %small complex residual of order e-15i sometimes remains

X=WL/(2*nw)-3.5*(n2/nw); %final answer: KL
end

X=X*10000;

if abs(X)<=1e-10
    X=0; %prevent numerical issues with tiny negative KL 
end

if X<0
    X=1e10; %in case of numerical problem 
end


end