function KL = klswbc(theta1,theta2,msel1,msel2)
%computes KL distance using only business cycle frequencies

%solve both models
[TT1,TC1,TEPS1,RC1] = swsolv(theta1,msel1);
[TT2,TC2,TEPS2,RC2] = swsolv(theta2,msel2);

%check whether both models have solutions
if isempty(TT1) || isempty(TT2)
    disp('empty solution: nonexistence/numerical issue') 
    KL=[];
    return
end

neq1=size(TT1,2); %number of equations - model 1
neq2=size(TT2,2); %number of equations - model 2



%the integral [-pi,pi] is approximated using n points
n=10000;
w=2*pi*(-(n/2-1):1:(n/2))'/n; %form vector of frequencies for approximation of the integral

%identify BC frequencies
ind1=find(w>=-pi/3 & w<=-pi/16);
ind2=find(w>=pi/16 & w<=pi/3);
idbc=[ind1;ind2];

%update to BC frequencies for computation
w=w(idbc);
n2=length(w);


dimy=size(TEPS1,2);

sp_mat1=zeros(dimy*n2,dimy); 
sp_mat2=sp_mat1;

%observables selection matrix
matB=eye(neq1);
matA=matB(18:24,:);

%compute model 1 spectrum over all frequencies

for i=1:n2; %loop computes spectra and stacks them in 1 big matrix
    exe=exp(-1i*w(i));
    
    fact1=matA*((eye(neq1) - TT1*exe)\TEPS1);
sp_mat1(((i-1)*dimy+1):(i*dimy),:)=(1/(2*pi))*(fact1*fact1'); 


end;

clear matA matB

%compute model 2 spectrum over all frequencies
matB=eye(neq2);
matA=matB(18:24,:);
for i=1:n2; %loop computes spectra and stacks them in 1 big matrix
    exe=exp(-1i*w(i));
    
    fact1=matA*((eye(neq2) - TT2*exe)\TEPS2);
sp_mat2(((i-1)*dimy+1):(i*dimy),:)=(1/(2*pi))*(fact1*fact1');


end;

%approximate the integral
term1=0;
term2=0;
for i=1:n2;
    
  term1=term1 - log(det(sp_mat2(((i-1)*dimy+1):(i*dimy),:)\sp_mat1(((i-1)*dimy+1):(i*dimy),:))); %second term in the KL


    term2=term2 + trace(sp_mat2(((i-1)*dimy+1):(i*dimy),:)\sp_mat1(((i-1)*dimy+1):(i*dimy),:));

end
WL=real(term1+term2); %small complex residual of order e-15i sometimes remains
KL=WL/(2*n)-3.5*(n2/n); %final answer: KL

end