function V = vfhsw(theta1,theta2,msel1,msel2)
%computes asymptotic variance for LR test for SW(2007)-type models

%solve both models
[TT1,TC1,TEPS1,RC1] = swsolv(theta1,msel1);
[TT2,TC2,TEPS2,RC2] = swsolv(theta2,msel2);

%check whether both models have solutions
if isempty(TT1) || isempty(TT2)
    disp('empty solution: nonexistence/numerical issue') 
    V=[];
    return
end
neq1=size(TT1,2); %number of equations - model 1
neq2=size(TT2,2); %number of equations - model 2

%observables selection matrix
matB=eye(neq1);
matA=matB(18:24,:);
tempeye=eye(neq1);

%the integral [-pi,pi] is approximated using n points
n=10000;
w=2*pi*(-(n/2-1):1:(n/2))'/n; %form vector of frequencies for approximation of the integral
dimy=size(TEPS1,2); %number of observables
sp_mat1=zeros(dimy*n,dimy); %blank for spectra at all frequencies in omi
sp_mat2=sp_mat1;

%compute model 1 spectrum over all frequencies

for i=1:n; %loop computes spectra and stacks them in 1 big matrix
    exe=exp(-1i*w(i));
    
    fact1=matA*((eye(neq1) - TT1*exe)\TEPS1);
sp_mat1(((i-1)*dimy+1):(i*dimy),:)=(1/(2*pi))*(fact1*fact1'); 


end;

clear matA matB

%compute model 2 spectrum over all frequencies
matB=eye(neq2);
matA=matB(18:24,:);
for i=1:n; %loop computes spectra and stacks them in 1 big matrix
    exe=exp(-1i*w(i));
    
    fact1=matA*((eye(neq2) - TT2*exe)\TEPS2);
sp_mat2(((i-1)*dimy+1):(i*dimy),:)=(1/(2*pi))*(fact1*fact1');


end;

%approximate the integral
vint=0; %initialize the quantity inside the integral
for i=1:n
    
    temp = (eye(dimy) - (sp_mat1(((i-1)*dimy+1):(i*dimy),:)/sp_mat2(((i-1)*dimy+1):(i*dimy),:)))^2; %(I-f_theta*inv(h_phi))
    
    vint = vint + trace(temp); %term inside the integral
end

V = real((1/(4*pi))*(2*pi/n)*vint); %final result. Take real part since very small complex residual may remain.

end